from vcCommand import *
from vcHelpers.Application import *
from datetime import datetime
import os, sys, subprocess, vcVector, vcMatrix
from PIL import Image

app = getApplication()

def OnOpenFolder(arg):
  os.startfile(rootFolder) 
  
def OnCreate(arg):
  app.SelectionManager.clear()
  
  camera = app.findCamera()
  coi = camera.Coi  
  eye = camera.Eye  
  
  ##################
  app.ProgressStatus = "Start ..."
  app.ProgressValue = 0.0
  
  # Create current folder
  currentTime = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
  currentFolder = "%s\\%s" % (rootFolder, currentTime)
  os.mkdir(currentFolder)  
  
  ##################
  app.ProgressStatus = "Resize and extract images ..."
  app.ProgressValue = 0.2
  
  # Resize
  videoFile = video.Value[8:]
  resizeVideoFile = "%s\\Resize.mp4" % (currentFolder)
  resizeCommand = "\"%s\" -loglevel quiet -stats -y -i \"%s\" -vf \"scale=%i:-1\" \"%s\"" % (ffmpegPath, videoFile, videoWidth.Value, resizeVideoFile)  
  subprocess.call(resizeCommand)

  # Extract image from video
  extractCommand = "\"%s\" -loglevel quiet -stats -y -i \"%s\" -vf fps=%i \"%s\\Image_%%04d.jpg\"" % (ffmpegPath, resizeVideoFile, fps.Value, currentFolder)  
  subprocess.call(extractCommand)
  
  ##################
  app.ProgressStatus = "Create component ..."
  app.ProgressValue = 0.4
  
  # Create component
  component = app.createComponent()
  component.Name = "VideoPlayer_%s" % currentTime
  component.BOMname = ""
  
  scale = component.createProperty(VC_REAL, "Scale")
  scale.Value = 10.0
  scale.ChangeOnRebuild = True
  
  frame = component.createProperty(VC_INTEGER, "Frame", VC_PROPERTY_STEP)
  frame.Value = 0  

  ##################
  app.ProgressStatus = "Add nodes and materials ..."
  app.ProgressValue = 0.6
  
  # Add material
  imageWidth = -1
  imageHeight = -1
  counter = 0
  for file in os.listdir(currentFolder):
    if file.endswith(".jpg"):          
      # Create material
      materialName = "VideoPlayer_%s_%s" % (currentTime, file[:-4])
      material = app.createMaterial(materialName) 
      app.addMaterial(material)     
      
      # Texture
      textureFile = "%s\\%s" % (currentFolder, file) 
      if imageWidth == -1 and imageHeight == -1:
        image = Image.open(textureFile)
        imageWidth, imageHeight = image.size
      texture = app.loadBitmap(textureFile)
      material.Texture = texture
      
      # Misc
      diffuseVector = vcVector.new(1,1,1)
      material.Diffuse = diffuseVector
      
      ambientVector = vcVector.new(0.8, 0.8, 0.8)
      material.Ambient = ambientVector
      
      childNode = component.createNode(VC_NODE, "Node_%i" % counter)
      childNode.JointType = VC_JOINTTYPE_CUSTOM
      childNode.Dof.Joint = "Frame==%i?Sx(Scale).Sy(Scale).Sz(Scale):Sx(0.0).Sy(0.0).Sz(0.0)" % counter      
      block = childNode.RootFeature.createFeature(VC_BLOCK, "Block_%i" % counter)
      block.Length = "%f {mm}" % imageWidth
      block.Width = "0.0 {mm}"
      block.Height = "%f {mm}" % imageHeight
      block.collapse()
      
      block = component.findFeature("Block_%i" % counter)
      geometry = block.Geometry.GeometrySets[0]
      for i in range(0, geometry.PointCount):
        vector = geometry.getPoint(i)
        u = (vector.X / imageWidth)
        v = (vector.Z / imageHeight)
        geometry.setTexCoord1(i, u, v)
      geometry.Material = material
      block.rebuild()
      counter += 1
  
  ##################
  app.ProgressStatus = "Add nodes and materials ..."
  app.ProgressValue = 0.8
  
  transform = component.RootFeature.createFeature(VC_TRANSFORM, "Transform")
  transform.Expression = "Sx(Scale).Sy(Scale).Sz(Scale)"
  wall = transform.createFeature(VC_BLOCK, "Block")
  wall.Length = "%f {mm}" % imageWidth
  wall.Width = "0.0 {mm}"
  wall.Height = "%f {mm}" % imageHeight
  wall.Material = app.findMaterial("black_matte")
  wallPM = wall.PositionMatrix
  wallPM.translateRel(0.0, 1.0, 0.0)
  wall.PositionMatrix = wallPM
  
  frame.StepValues = list(range(0, counter)) 
  
  script = component.createBehaviour(VC_SCRIPT, "PythonScript")
  script.Script = '''
from vcScript import *

comp = getComponent()
frame = comp.getProperty("Frame")

def OnRun():
  while True:
    for currentIndex in frame.StepValues:
      comp.Frame = currentIndex
      delay(1.0/%i)   
''' % fps.Value
  component.rebuild()
    
  ##################
  app.ProgressStatus = "Done"
  app.ProgressValue = 1.0
  
  app.SelectionManager.setSelection(component)
  camera.Coi = component.WorldPositionMatrix.P  
  camera.Eye= eye + (camera.Coi - coi)  
  
  app.render()
  
def OnStart():
  global diag, video, videoWidth, fps
  
  diag = getDialog("VideoPlayer")
  video = diag.addProperty("Video", VC_URI, None)
  fps = diag.addProperty("FPS", VC_INTEGER, None)
  fps.Value = 25
  videoWidth = diag.addProperty("VideoWidth", VC_INTEGER, None)
  videoWidth.Value = 640
  create = diag.addProperty("Create", VC_BUTTON, OnCreate)
  openFolder = diag.addProperty("Folder", VC_BUTTON, OnOpenFolder)
  
  # Check root folder
  global rootFolder
  rootFolder = getCommandPath().split("\\")[:-1]    
  rootFolder.append("Images")
  rootFolder = "\\".join(rootFolder)
  rootFolder = rootFolder[8:]
  if not os.path.exists(rootFolder):
    os.makedirs(rootFolder)
  
  # Internal ffmpeg
  global ffmpegPath
  ffmpegPath = sys.executable.split("\\")[:-1]  
  ffmpegPath.append("ffmpeg")
  ffmpegPath.append("ffmpeg.exe")
  ffmpegPath = "\\".join(ffmpegPath)
    
  diag.show()
  